@@ -14,7 +14,11 @@ from botorch import (
     test_functions,
 )
 from botorch.cross_validation import batch_cross_validation
-from botorch.fit import fit_fully_bayesian_model_nuts, fit_gpytorch_model
+from botorch.fit import (
+    fit_fully_bayesian_model_nuts,
+    fit_gpytorch_mll,
+    fit_gpytorch_model,
+)
 from botorch.generation.gen import (
     gen_candidates_scipy,
     gen_candidates_torch,
@@ -34,6 +38,7 @@ __all__ = [
     "batch_cross_validation",
     "exceptions",
     "fit_fully_bayesian_model_nuts",
+    "fit_gpytorch_mll",
     "fit_gpytorch_model",
     "gen_candidates_scipy",
     "gen_candidates_torch",
