@@ -210,7 +210,7 @@ class ExpectationPosteriorTransform(PosteriorTransform):
         Returns:
             An `m`-outcome joint posterior over `q` expectations.
         """
-        org_mvn = posterior.mvn
+        org_mvn = posterior.distribution
         if getattr(org_mvn, "_interleaved", False):
             raise UnsupportedError(
                 "`ExpectationPosteriorTransform` does not support "
@@ -252,7 +252,7 @@ class ExpectationPosteriorTransform(PosteriorTransform):
             new_mvn = MultitaskMultivariateNormal(
                 new_loc, to_linear_operator(new_cov), interleaved=False
             )
-        return GPyTorchPosterior(mvn=new_mvn)
+        return GPyTorchPosterior(distribution=new_mvn)
 
 
 class MCAcquisitionObjective(Module, ABC):
@@ -453,7 +453,7 @@ class ConstrainedMCObjective(GenericMCObjective):
         objective: Callable[[Tensor, Optional[Tensor]], Tensor],
         constraints: List[Callable[[Tensor], Tensor]],
         infeasible_cost: Union[Tensor, float] = 0.0,
-        eta: float = 1e-3,
+        eta: Union[Tensor, float] = 1e-3,
     ) -> None:
         r"""
         Args:
@@ -468,11 +468,17 @@ class ConstrainedMCObjective(GenericMCObjective):
             infeasible_cost: The cost of a design if all associated samples are
                 infeasible.
             eta: The temperature parameter of the sigmoid function approximating
-                the constraint.
+                the constraint. Can be either a float or a 1-dim tensor. In case
+                of a float the same eta is used for every constraint in
+                constraints. In case of a tensor the length of the tensor must
+                match the number of provided constraints. The i-th constraint is
+                then estimated with the i-th eta value.
         """
         super().__init__(objective=objective)
         self.constraints = constraints
-        self.register_buffer("eta", torch.as_tensor(eta))
+        if type(eta) != Tensor:
+            eta = torch.full((len(constraints),), eta)
+        self.register_buffer("eta", eta)
         self.register_buffer("infeasible_cost", torch.as_tensor(infeasible_cost))
 
     def forward(self, samples: Tensor, X: Optional[Tensor] = None) -> Tensor:
@@ -528,7 +534,7 @@ class LearnedObjective(MCAcquisitionObjective):
             sampler: Sampler for the preference model to account for uncertainty in
                 preferece when calculating the objective; it's not the one used
                 in MC acquisition functions. If None,
-                it uses `IIDNormalSampler(num_samples=1)`.
+                it uses `IIDNormalSampler(sample_shape=torch.Size([1]))`.
         """
         super().__init__()
         self.pref_model = pref_model
@@ -537,7 +543,7 @@ class LearnedObjective(MCAcquisitionObjective):
             self.sampler = None
         else:
             if sampler is None:
-                self.sampler = IIDNormalSampler(num_samples=1)
+                self.sampler = IIDNormalSampler(sample_shape=torch.Size([1]))
             else:
                 self.sampler = sampler
 
