@@ -19,6 +19,7 @@ import torch
 from botorch.acquisition import AcquisitionFunction
 from botorch.exceptions.warnings import OptimizationWarning
 from botorch.generation.utils import _remove_fixed_features_from_optimization
+from botorch.logging import _get_logger
 from botorch.optim.parameter_constraints import (
     _arrayify,
     make_scipy_bounds,
@@ -29,9 +30,12 @@ from botorch.optim.parameter_constraints import (
 from botorch.optim.stopping import ExpMAStoppingCriterion
 from botorch.optim.utils import _filter_kwargs, columnwise_clamp, fix_features
 from scipy.optimize import minimize
+from scipy.optimize.optimize import OptimizeResult
 from torch import Tensor
 from torch.optim import Optimizer
 
+logger = _get_logger()
+
 
 def gen_candidates_scipy(
     initial_conditions: Tensor,
@@ -68,7 +72,9 @@ def gen_candidates_scipy(
         options: Options used to control the optimization including "method"
             and "maxiter". Select method for `scipy.minimize` using the
             "method" key. By default uses L-BFGS-B for box-constrained problems
-            and SLSQP if inequality or equality constraints are present.
+            and SLSQP if inequality or equality constraints are present. If
+            `with_grad=False`, then we use a two-point finite difference estimate
+            of the gradient.
         fixed_features: This is a dictionary of feature indices to values, where
             all generated candidates will have features fixed to these values.
             If the dictionary value is None, then that feature will just be
@@ -95,6 +101,7 @@ def gen_candidates_scipy(
             )
     """
     options = options or {}
+    options = {**options, "maxiter": options.get("maxiter", 2000)}
 
     # if there are fixed features we may optimize over a domain of lower dimension
     reduced_domain = False
@@ -154,34 +161,48 @@ def gen_candidates_scipy(
         equality_constraints=equality_constraints,
     )
 
-    def f_np_wrapper(x: np.ndarray, f: Callable):
-        """Given a torch callable, compute value + grad given a numpy array."""
-        if np.isnan(x).any():
-            raise RuntimeError(
-                f"{np.isnan(x).sum()} elements of the {x.size} element array "
-                f"`x` are NaN."
+    with_grad = options.get("with_grad", True)
+    if with_grad:
+
+        def f_np_wrapper(x: np.ndarray, f: Callable):
+            """Given a torch callable, compute value + grad given a numpy array."""
+            if np.isnan(x).any():
+                raise RuntimeError(
+                    f"{np.isnan(x).sum()} elements of the {x.size} element array "
+                    f"`x` are NaN."
+                )
+            X = (
+                torch.from_numpy(x)
+                .to(initial_conditions)
+                .view(shapeX)
+                .contiguous()
+                .requires_grad_(True)
             )
-        X = (
-            torch.from_numpy(x)
-            .to(initial_conditions)
-            .view(shapeX)
-            .contiguous()
-            .requires_grad_(True)
-        )
-        X_fix = fix_features(X, fixed_features=fixed_features)
-        loss = f(X_fix).sum()
-        # compute gradient w.r.t. the inputs (does not accumulate in leaves)
-        gradf = _arrayify(torch.autograd.grad(loss, X)[0].contiguous().view(-1))
-        if np.isnan(gradf).any():
-            msg = (
-                f"{np.isnan(gradf).sum()} elements of the {x.size} element "
-                "gradient array `gradf` are NaN. This often indicates numerical issues."
-            )
-            if initial_conditions.dtype != torch.double:
-                msg += " Consider using `dtype=torch.double`."
-            raise RuntimeError(msg)
-        fval = loss.item()
-        return fval, gradf
+            X_fix = fix_features(X, fixed_features=fixed_features)
+            loss = f(X_fix).sum()
+            # compute gradient w.r.t. the inputs (does not accumulate in leaves)
+            gradf = _arrayify(torch.autograd.grad(loss, X)[0].contiguous().view(-1))
+            if np.isnan(gradf).any():
+                msg = (
+                    f"{np.isnan(gradf).sum()} elements of the {x.size} element "
+                    "gradient array `gradf` are NaN. "
+                    "This often indicates numerical issues."
+                )
+                if initial_conditions.dtype != torch.double:
+                    msg += " Consider using `dtype=torch.double`."
+                raise RuntimeError(msg)
+            fval = loss.item()
+            return fval, gradf
+
+    else:
+
+        def f_np_wrapper(x: np.ndarray, f: Callable):
+            X = torch.from_numpy(x).to(initial_conditions).view(shapeX).contiguous()
+            with torch.no_grad():
+                X_fix = fix_features(X=X, fixed_features=fixed_features)
+                loss = f(X_fix).sum()
+            fval = loss.item()
+            return fval
 
     if nonlinear_inequality_constraints:
         # Make sure `batch_limit` is 1 for now.
@@ -205,29 +226,18 @@ def gen_candidates_scipy(
         args=(f,),
         x0=x0,
         method=options.get("method", "SLSQP" if constraints else "L-BFGS-B"),
-        jac=True,
+        jac=with_grad,
         bounds=bounds,
         constraints=constraints,
         callback=options.get("callback", None),
-        options={k: v for k, v in options.items() if k not in ["method", "callback"]},
+        options={
+            k: v
+            for k, v in options.items()
+            if k not in ["method", "callback", "with_grad"]
+        },
     )
+    _process_scipy_result(res=res, options=options)
 
-    if "success" not in res.keys() or "status" not in res.keys():
-        with warnings.catch_warnings():
-            warnings.simplefilter("always", category=OptimizationWarning)
-            warnings.warn(
-                "Optimization failed within `scipy.optimize.minimize` with no "
-                "status returned to `res.`",
-                OptimizationWarning,
-            )
-    elif not res.success:
-        with warnings.catch_warnings():
-            warnings.simplefilter("always", category=OptimizationWarning)
-            warnings.warn(
-                f"Optimization failed within `scipy.optimize.minimize` with status "
-                f"{res.status}.",
-                OptimizationWarning,
-            )
     candidates = fix_features(
         X=torch.from_numpy(res.x).to(initial_conditions).reshape(shapeX),
         fixed_features=fixed_features,
@@ -399,3 +409,37 @@ def get_best_candidates(batch_candidates: Tensor, batch_values: Tensor) -> Tenso
     """
     best = torch.argmax(batch_values.view(-1), dim=0)
     return batch_candidates[best]
+
+
+def _process_scipy_result(res: OptimizeResult, options: Dict[str, Any]) -> None:
+    r"""Process scipy optimization result to produce relevant logs and warnings."""
+    if "success" not in res.keys() or "status" not in res.keys():
+        with warnings.catch_warnings():
+            warnings.simplefilter("always", category=OptimizationWarning)
+            warnings.warn(
+                "Optimization failed within `scipy.optimize.minimize` with no "
+                "status returned to `res.`",
+                OptimizationWarning,
+            )
+    elif not res.success:
+        if (
+            "ITERATIONS REACHED LIMIT" in res.message
+            or "Iteration limit reached" in res.message
+        ):
+            logger.info(
+                "`scipy.minimize` exited by reaching the iteration limit of "
+                f"`maxiter: {options.get('maxiter')}`."
+            )
+        elif "EVALUATIONS EXCEEDS LIMIT" in res.message:
+            logger.info(
+                "`scipy.minimize` exited by reaching the function evaluation limit of "
+                f"`maxfun: {options.get('maxfun')}`."
+            )
+        else:
+            with warnings.catch_warnings():
+                warnings.simplefilter("always", category=OptimizationWarning)
+                warnings.warn(
+                    f"Optimization failed within `scipy.optimize.minimize` with status "
+                    f"{res.status}.",
+                    OptimizationWarning,
+                )
